@@ -1,15 +1,16 @@
-//! Stateful reducers which maintain distinct count sketches,
-//! aimed at servicing the `dsrs` command-line tool for deduplicating
-//! byte lines of input.
+//! Stateful reducers which maintain distinct count and heavy
+//! hitters sketches, aimed at servicing the `dsrs` command-line tool
+//! for deduplicating byte lines of input.
 
 use std::collections::HashMap;
+use std::convert::TryInto;
 use std::str;
 
 use base64;
 use memchr;
 
 use crate::stream_reducer::LineReducer;
-use crate::{CpcSketch, CpcUnion};
+use crate::{CpcSketch, CpcUnion, HhSketch};
 
 pub struct Counter {
     sketch: CpcSketch,
@@ -146,3 +147,56 @@ impl KeyedMerger {
             .map(|(key, mrgr)| (key.as_ref(), mrgr.counter()))
     }
 }
+
+pub struct HeavyHitter {
+    sketch: HhSketch,
+    k: u64
+}
+
+// https://users.rust-lang.org/t/logarithm-of-integers/8506/5
+
+fn log2_floor(x: u64) -> usize {
+    const fn num_bits<T>() -> usize { std::mem::size_of::<T>() * 8 }
+    assert!(x > 0);
+    num_bits::<u64>() - x.leading_zeros() as usize - 1
+}
+
+impl HeavyHitter {
+
+    /// Creates a new heavy hitter sketch targeting elements in the top-k
+    /// by reserving O(k) space.
+    pub fn new( k: u64) -> Self {
+        let lg2_k_with_room = log2_floor(k as u64).max(1) + 2;
+        Self {
+            sketch: HhSketch::new(lg2_k_with_room.try_into().unwrap()),
+            k
+        }
+    }
+    
+    /// Serializes to base64 string with no newlines or `=` padding.
+    pub fn serialize(&self) -> String {
+        unimplemented!()
+    }
+
+    /// Deserializes from base64 string with no newlines or `=` padding.
+    pub fn deserialize(_s: &str) -> Result<Self, base64::DecodeError> {
+        unimplemented!()
+    }
+
+    /// Returns pairs (heavy hitter slice, estimate of count size)
+    pub fn estimate(&self) -> impl Iterator<Item = (&[u8], u64)> {
+        let mut v = self.sketch.estimate_no_fn();
+        v.sort_by_key(|row| row.ub);
+        v
+            .into_iter()
+            .rev()
+            .take(self.k as usize)
+            .map(|row| (row.key, row.ub))
+    }
+}
+
+impl LineReducer for HeavyHitter {
+    fn read_line(&mut self, line: &[u8]) {
+        self.sketch.update(line, 1);
+    }
+}
